package dev.fyloz.colorrecipesexplorer.config.security.filters

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.properties.CreSecurityProperties
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import dev.fyloz.colorrecipesexplorer.utils.addCookie
import org.springframework.http.HttpMethod
import org.springframework.security.core.Authentication
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter
import org.springframework.security.web.util.matcher.AntPathRequestMatcher
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

abstract class JwtAuthenticationFilter(
    filterProcessesUrl: String,
    private val securityProperties: CreSecurityProperties,
    private val jwtLogic: JwtLogic
) :
    AbstractAuthenticationProcessingFilter(
        AntPathRequestMatcher(filterProcessesUrl, HttpMethod.POST.toString())
    ) {
    override fun successfulAuthentication(
        request: HttpServletRequest,
        response: HttpServletResponse,
        chain: FilterChain,
        auth: Authentication
    ) {
        val userDetails = auth.principal as UserDetails
        val token = jwtLogic.buildJwt(userDetails)

        addAuthorizationHeaders(response, token)
        addAuthorizationCookie(response, token)

        afterSuccessfulAuthentication(userDetails)
    }

    protected abstract fun afterSuccessfulAuthentication(userDetails: UserDetails)

    private fun addAuthorizationHeaders(response: HttpServletResponse, token: String) {
        response.addHeader(Constants.HeaderNames.ACCESS_CONTROL_EXPOSE_HEADERS, Constants.HeaderNames.AUTHORIZATION)
        response.addHeader(Constants.HeaderNames.AUTHORIZATION, "$BEARER_TOKEN_PREFIX $token")
    }

    private fun addAuthorizationCookie(response: HttpServletResponse, token: String) {
        response.addCookie(Constants.CookieNames.AUTHORIZATION, BEARER_TOKEN_PREFIX + token) {
            httpOnly = AUTHORIZATION_COOKIE_HTTP_ONLY
            sameSite = AUTHORIZATION_COOKIE_SAME_SITE
            secure = !Constants.DEBUG_MODE
            maxAge = securityProperties.jwtDuration / 1000
            path = AUTHORIZATION_COOKIE_PATH
        }
    }

    companion object {
        private const val AUTHORIZATION_COOKIE_HTTP_ONLY = true
        private const val AUTHORIZATION_COOKIE_SAME_SITE = true
        private const val AUTHORIZATION_COOKIE_PATH = Constants.ControllerPaths.BASE_PATH

        private const val BEARER_TOKEN_PREFIX = "Bearer"
    }
}