package dev.fyloz.colorrecipesexplorer.config.security.filters

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDetails
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import dev.fyloz.colorrecipesexplorer.logic.account.UserDetailsLogic
import io.jsonwebtoken.ExpiredJwtException
import org.springframework.security.authentication.AuthenticationManager
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.web.authentication.www.BasicAuthenticationFilter
import org.springframework.web.util.WebUtils
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

class JwtAuthorizationFilter(
    private val jwtLogic: JwtLogic,
    authenticationManager: AuthenticationManager,
    private val userDetailsLogic: UserDetailsLogic
) : BasicAuthenticationFilter(authenticationManager) {
    override fun doFilterInternal(request: HttpServletRequest, response: HttpServletResponse, chain: FilterChain) {
        fun tryLoginFromBearer(): Boolean {
            val authorizationCookie = WebUtils.getCookie(request, Constants.HeaderNames.AUTHORIZATION)
            // Check for an authorization token cookie or header
            val authorizationToken = if (authorizationCookie != null)
                authorizationCookie.value
            else
                request.getHeader(Constants.HeaderNames.AUTHORIZATION)

            // An authorization token is valid if it starts with "Bearer", is not expired and is not blacklisted
            if (authorizationToken != null && authorizationToken.startsWith("Bearer") && authorizationToken !in blacklistedJwtTokens) {
                val authenticationToken = getAuthentication(authorizationToken) ?: return false
                SecurityContextHolder.getContext().authentication = authenticationToken
                return true
            }
            return false
        }

        fun tryLoginFromDefaultGroupCookie() {
            val defaultGroupCookie = WebUtils.getCookie(request, defaultGroupCookieName)
            if (defaultGroupCookie != null) {
                val authenticationToken = getAuthenticationToken(defaultGroupCookie.value)
                SecurityContextHolder.getContext().authentication = authenticationToken
            }
        }

        if (!tryLoginFromBearer())
            tryLoginFromDefaultGroupCookie()

        chain.doFilter(request, response)
    }

    private fun getAuthentication(token: String): UsernamePasswordAuthenticationToken? {
        return try {
            val user = jwtLogic.parseJwt(token.replace("Bearer", ""))
            getAuthenticationToken(user)
        } catch (_: ExpiredJwtException) {
            null
        }
    }

    private fun getAuthenticationToken(user: UserDetails) =
        UsernamePasswordAuthenticationToken(user.id, null, user.authorities)

    private fun getAuthenticationToken(userId: Long): UsernamePasswordAuthenticationToken? = try {
        val userDetails = userDetailsLogic.loadUserById(userId)
        UsernamePasswordAuthenticationToken(userDetails.username, null, userDetails.authorities)
    } catch (_: NotFoundException) {
        null
    }

    private fun getAuthenticationToken(userId: String) =
        getAuthenticationToken(userId.toLong())
}
